@@ -7,6 +7,45 @@ from sklearn.preprocessing import MultiLabelBinarizer, OneHotEncoder, LabelEncod
 from sklearn.preprocessing import PolynomialFeatures
 
 
+class LabelEncoderExt(object):
+    def __init__(self):
+        """
+        It differs from LabelEncoder by handling new classes and providing a value for it [Unknown]
+        Unknown will be added in fit and transform will take care of new item. It gives unknown class id
+        """
+        self.label_encoder = LabelEncoder()
+        # self.classes_ = self.label_encoder.classes_
+
+    def fit(self, data_list):
+        """
+        This will fit the encoder for all the unique values and introduce unknown value
+        :param data_list: A list of string
+        :return: self
+        """
+        self.label_encoder = self.label_encoder.fit(list(data_list) + ['Unknown'])
+        self.classes_ = self.label_encoder.classes_
+
+        return self
+
+    def transform(self, data_list):
+        """
+        This will transform the data_list to id list where the new values get assigned to Unknown class
+        :param data_list:
+        :return:
+        """
+        new_data_list = list(data_list)
+        for unique_item in np.unique(data_list):
+            if unique_item not in self.label_encoder.classes_:
+                new_data_list = ['Unknown' if x==unique_item else x for x in new_data_list]
+
+        return self.label_encoder.transform(new_data_list)
+
+    def fit_transform(self, data_list):
+
+        self.fit(data_list)
+        return self.transform(data_list)
+
+
 def upload_blob(bucket_name, source_file_name, destination_blob_name):
     """Uploads a file to the bucket."""
     storage_client = storage.Client()
@@ -104,7 +143,7 @@ def create_quantile_multilabel_binarizer():
 
 def create_one_hot_encoder_for_states():
     state_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    state_lbe = LabelEncoder()
+    state_lbe = LabelEncoderExt()
     client = bigquery.Client()
     states = []
 
@@ -187,7 +226,7 @@ def create_one_hot_encoder_for_states():
 def create_one_hot_encoder_for_cities():
 
     city_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    city_lbe = LabelEncoder()
+    city_lbe = LabelEncoderExt()
     client = bigquery.Client()
     state_cities = []
 
@@ -521,7 +560,7 @@ def create_one_hot_encoder_for_cities():
 
 def create_one_hot_encoder_load_source():
     source_ohe = OneHotEncoder(handle_unknown='ignore')
-    source_lbe = LabelEncoder()
+    source_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
@@ -553,7 +592,7 @@ def create_one_hot_encoder_load_source():
 def create_one_hot_encoder_email():
 
     email_ohe = OneHotEncoder(handle_unknown='ignore')
-    email_lbe = LabelEncoder()
+    email_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
@@ -694,7 +733,7 @@ def create_polynomial_features(feature_name_list, input_df):
 
 def create_one_hot_encoder_for_user_zip_code():
     zip_code_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    zip_code_lbe = LabelEncoder()
+    zip_code_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
@@ -723,7 +762,7 @@ def create_one_hot_encoder_for_user_zip_code():
 
 def create_one_hot_encoder_for_user_zip_code():
     zip_code_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    zip_code_lbe = LabelEncoder()
+    zip_code_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
@@ -752,7 +791,7 @@ def create_one_hot_encoder_for_user_zip_code():
 
 def create_one_hot_encoder_for_user_dot():
     dot_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    dot_lbe = LabelEncoder()
+    dot_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
@@ -781,7 +820,7 @@ def create_one_hot_encoder_for_user_dot():
 
 def create_one_hot_encoder_for_user_email():
     email_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    email_lbe = LabelEncoder()
+    email_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
@@ -811,7 +850,7 @@ def create_one_hot_encoder_for_user_email():
 
 def create_one_hot_encoder_for_user_hos_cycle():
     hos_cycle_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    hos_cycle_lbe = LabelEncoder()
+    hos_cycle_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
@@ -841,7 +880,7 @@ def create_one_hot_encoder_for_user_hos_cycle():
 
 def create_one_hot_encoder_for_user_hos_home_timezone():
     hos_home_timezone_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    hos_home_timezone_lbe = LabelEncoder()
+    hos_home_timezone_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
@@ -871,7 +910,7 @@ def create_one_hot_encoder_for_user_hos_home_timezone():
 
 def create_one_hot_encoder_for_user_type():
     user_type_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    user_type_lbe = LabelEncoder()
+    user_type_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
@@ -901,7 +940,7 @@ def create_one_hot_encoder_for_user_type():
 
 def create_one_hot_encoder_for_user_mc_number():
     mc_number_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    mc_number_lbe = LabelEncoder()
+    mc_number_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
@@ -931,7 +970,7 @@ def create_one_hot_encoder_for_user_mc_number():
 
 def create_one_hot_encoder_for_user_channel():
     channel_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    channel_lbe = LabelEncoder()
+    channel_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
@@ -961,7 +1000,7 @@ def create_one_hot_encoder_for_user_channel():
 
 def create_one_hot_encoder_for_safer_safety_rating():
     safer_result_safety_rating_ohe = OneHotEncoder(handle_unknown='ignore', sparse=True)
-    safer_result_safety_rating_lbe = LabelEncoder()
+    safer_result_safety_rating_lbe = LabelEncoderExt()
     client = bigquery.Client()
 
     query = """
